<?php
/**
 * @file migrate_parcel.module
 * Contains UI page callbacks and forms.
 */

/**
 * Form builder for creating an exported parcel.
 */
function migrate_parcel_export_form($form, &$form_state) {
  $entity_info = entity_get_info();

  // Build grouped options for the select form element.
  $options = array();
  foreach ($entity_info as $entity_type => $entity_data) {
    $options[$entity_data['label']] = array();
    foreach ($entity_data['bundles'] as $bundle_name => $bundle_data) {
      $options[$entity_data['label']][$entity_type . ':' . $bundle_name] = $entity_data['label'] . ': ' . $bundle_data['label'];
    }
  }

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Machine name'),
    '#description' => t('The machine name for the migration.'),
    '#required' => TRUE,
  );
  $form['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Description'),
    '#required' => TRUE,
  );

  $form['export_bundles'] = array(
    '#type' => 'select',
    '#title' => t('Entities to export'),
    '#description' => t('Select a bundle. All entities of this bundle will be exported.'),
    '#options' => $options,
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Export parcel'),
    '#weight' => 40,
  );

  return $form;
}

/**
 * Validation handler.
 */
function migrate_parcel_export_form_validate($form, &$form_state) {
  if (preg_match('@[^a-zA-Z0-9]@', $form_state['values']['name'])) {
    form_set_error('name', t('Migration machine name may only contain uppercase and lowercase letters and numbers.'));
  }
}

/**
 * Submit handler.
 */
function migrate_parcel_export_form_submit($form, &$form_state) {
  $export_bundle = $form_state['values']['export_bundles'];

  list($entity_type, $bundle_name) = explode(':', $export_bundle);
  $machine_name = $form_state['values']['name'];

  $info = array(
    'machine_name'      => $machine_name,
    'description'       => $form_state['values']['description'],
    'entity'       => $entity_type,
    'bundle'       => $bundle_name,
    // @TODO: configurable filepath & making a subdirectory!
    'directory'         => "public://migrate_parcel/$machine_name"
  );
  // @TODO: put it in "sites/all/parcels/$machine_name/$machine_name.csv"?

  $batch = array(
    'operations' => array(
      array('migrate_parcel_batch_process_prepare', array($info)),
      array('migrate_parcel_batch_process_info', array($info)),
      array('migrate_parcel_batch_process_csv', array($info)),
    ),
    'finished' => 'migrate_parcel_batch_finished',
    'title' => t('Processing entities'),
    'init_message' => t('Export is starting.'),
    'progress_message' => t('Processed @current out of @total.'),
    'error_message' => t('Entity export has encountered an error.'),
    'file' => drupal_get_path('module', 'migrate_parcel') . '/migrate_parcel.batch.inc',
  );
  batch_set($batch);
}

/**
 * Update form.
 */
function migrate_parcel_update_form($form, &$form_state) {
  $entity_info = entity_get_info();
  $migrations = migrate_parcel_get_migrations();

  $options = array();
  foreach ($migrations as $migration) {
    $options[$migration['machine_name']] = t("@name (@entity: @bundle)", array(
      '@name' => $migration['machine_name'],
      '@entity' => $entity_info[$migration['entity']]['label'],
      '@bundle' => $entity_info[$migration['entity']]['bundles'][$migration['bundle']]['label'],
    ));
  }
  $form['#migrations'] = $migrations;

  $form['parcels'] = array(
    '#type' => 'radios',
    '#title' => t('Parcels to update'),
    '#description' => t('Select an entity parcel to update.'),
    '#options' => $options,
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update parcel'),
    '#weight' => 40,
  );

  return $form;
}

/**
 * Submit handler for the parcel update form.
 */
function migrate_parcel_update_form_submit($form, &$form_state) {
  $parcel_name = $form_state['values']['parcels'];
  $migrations = $form['#migrations'];

  // We can just take the parcel info array as our info for the batch.
  $info = $migrations[$parcel_name];
  // Though we need to add in the folder path.
  $info['directory'] = dirname($info['source_file']);

  $batch = array(
    'operations' => array(
      // This writes direct to the existing CSV location.
      // That's okay as we're not writing code, just data.
      array('migrate_parcel_batch_process_csv', array($info)),
    ),
    'finished' => 'migrate_parcel_batch_finished',
    'title' => t('Processing entities'),
    'init_message' => t('Export is starting.'),
    'progress_message' => t('Processed @current out of @total.'),
    'error_message' => t('Entity export has encountered an error.'),
    'file' => drupal_get_path('module', 'migrate_parcel') . '/migrate_parcel.batch.inc',
  );
  batch_set($batch);
}
